@@ -124,11 +124,10 @@ getMeta <- function(site = "heathrow", lat = NA, lon = NA,
         
     }
     
-    
+    dat <- dplyr::rename(meta, latitude = LAT, longitude = LON)   
   
   if (plot) {
-    
-    dat <- dplyr::rename(meta, latitude = LAT, longitude = LON) 
+  
     
     if (!"dist" %in% names(dat)) dat$dist <- NA
     
@@ -162,12 +161,8 @@ getMetaLive <- function(...) {
 
     ## downloads the whole thing fresh
        
-    ## use RCurl 
-    bin <- getBinaryURL("ftp://ftp.ncdc.noaa.gov/pub/data/noaa/isd-history.csv",
-                        ssl.verifypeer = FALSE)
-    tmp <- tempfile()
-    writeBin(bin, tmp)
-    meta <-  read.csv(tmp, header = FALSE, skip = 21)
+    url <- "https://www1.ncdc.noaa.gov/pub/data/noaa/isd-history.csv"
+    meta <- suppressMessages(read.csv(url, skip = 21))
 
     ## names in the meta file
     names(meta) <- c("USAF", "WBAN","STATION", "CTRY", "ST", "CALL", "LAT",
